/*
 *  Kylin national secret algorithm module
 *
 *  This file contains the ecies data structures
 *  and functions declared.
 *
 *  Authors:  YangZhao <yangzhao@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#ifndef HEADER_ECIES_H
#define HEADER_ECIES_H

#include <openssl/ec.h>
#include <openssl/err.h>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/asn1.h>
#include <openssl/objects.h>
#include <gm/kdf2.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
ECIESAlgorithmSet ALGORITHM ::= {
	{OID ecies-recommendedParameters} |
	{OID ecies-specifiedParameters PARMS ECIESParameters},
	... -- Future combinations may be added
	{OID ecies-with-x9-63-sha1-aes128-cbc-hmac}
	{OID ecies-with-x9-63-sha256-aes128-cbc-hmac}
	{OID ecies-with-x9-63-sha256-aes192-cbc-hmac}
	{OID ecies-with-x9-63-sha512-aes256-cbc-hmac}
	{OID ecies-with-x9-63-sha256-xor-hmac}
}
*/

typedef struct ecies_params_st {
	/*
	KDFSet ALGORITHM ::= {
		{ OID x9-63-kdf PARMS HashAlgorithm } |
		{ OID nist-concatenation-kdf PARMS HashAlgorithm } |
		{ OID tls-kdf PARMS HashAlgorithm } |
		{ OID ikev2-kdf PARMS HashAlgorithm }
		... -- Future combinations may be added
	}
	*/
	int kdf_nid;
	const EVP_MD *kdf_md;

	/*
	SYMENCSet ALGORITHM ::= {
		{ OID xor-in-ecies } |
		{ OID tdes-cbc-in-ecies } |
		{ OID aes128-cbc-in-ecies } |
		{ OID aes192-cbc-in-ecies } |
		{ OID aes256-cbc-in-ecies } |
		{ OID aes128-ctr-in-ecies } |
		{ OID aes192-ctr-in-ecies } |
		{ OID aes256-ctr-in-ecies } ,
		... -- Future combinations may be added
	}
	*/
	int enc_nid;

	/*
	MACSet ALGORITHM ::= {
		{ OID hmac-full-ecies PARMS HashAlgorithm} |
		{ OID hmac-half-ecies PARMS HashAlgorithm} |
		{ OID cmac-aes128-ecies } |
		{ OID cmac-aes192-ecies } |
		{ OID cmac-aes256-ecies } ,
		... -- Future combinations may be added
	}
	*/
	int mac_nid;
	const EVP_MD *hmac_md;

} ECIES_PARAMS;

int ECIES_PARAMS_init_with_recommended(ECIES_PARAMS *param);
int ECIES_PARAMS_init_with_type(ECIES_PARAMS *param, int type);
KDF_FUNC ECIES_PARAMS_get_kdf(const ECIES_PARAMS *param);
int ECIES_PARAMS_get_enc(const ECIES_PARAMS *param, size_t inlen,
	const EVP_CIPHER **enc_cipher, size_t *enckeylen, size_t *ciphertextlen);
int ECIES_PARAMS_get_mac(const ECIES_PARAMS *param,
	const EVP_MD **hmac_md, const EVP_CIPHER **cmac_cipher,
	unsigned int *mackeylen, unsigned int *maclen);

int i2d_ECIESParameters(const ECIES_PARAMS *param, unsigned char **out);
ECIES_PARAMS *d2i_ECIESParameters(ECIES_PARAMS **param,
	const unsigned char **in, long len);

typedef struct ecies_ciphertext_value_st ECIES_CIPHERTEXT_VALUE;
DECLARE_ASN1_FUNCTIONS(ECIES_CIPHERTEXT_VALUE)


ECIES_CIPHERTEXT_VALUE *ECIES_do_encrypt(const ECIES_PARAMS *param,
	const unsigned char *in, size_t inlen, EC_KEY *ec_key);
int ECIES_do_decrypt(const ECIES_PARAMS *param, const ECIES_CIPHERTEXT_VALUE *in,
	unsigned char *out, size_t *outlen, EC_KEY *ec_key);
int ECIES_encrypt(int type, const unsigned char *in, size_t inlen,
	unsigned char *out, size_t *outlen, EC_KEY *ec_key);
int ECIES_decrypt(int type, const unsigned char *in, size_t inlen,
	unsigned char *out, size_t *outlen, EC_KEY *ec_key);
#define ECIES_encrypt_with_recommended(in,inlen,out,outlen,ec_key) \
	ECIES_encrypt(NID_ecies_with_x9_63_sha256_xor_hmac,in,inlen,out,outlen,ec_key)
#define ECIES_decrypt_with_recommended(in,inlen,out,outlen,ec_key) \
	ECIES_decrypt(NID_ecies_with_x9_63_sha256_xor_hmac,in,inlen,out,outlen,ec_key)


#ifdef __cplusplus
}
#endif
#endif
